"""ui.py- base class for all UIs.
inherit ALL UI classes from this, as it wil handle the codes returned
from each item that is run.
"""
import sys


class UI:
    """base class for UIs.
    the method `process` should always be called after every
    item run. all other methods should be overridden.
    """
    codes = {
        0: lambda: None,
        1: sys.exit,
    }

    def __init__(self, items):
        self.items = items

    def process(self, code):
        """proccesses error code and runs proper function."""
        self.codes[code]()

    def run(self):
        """runs all items in proper UI"""
        pass



"""CLI: command line interface installer.
you will create a CLI object, and pass it to installer.
"""

import sys
import colorama

_MS_WINDOWS = sys.platform == 'win32'

try:
    from .uiutils._cli_progress_bar import ProgressBar
    from .ui import UI

except SystemError:
    from uiutils._cli_progress_bar import ProgressBar
    from ui import UI


if _MS_WINDOWS:
    import msvcrt
    _getche = msvcrt.getwche
    _getch = msvcrt.getwch

else:
    import tty
    import termios

    def _getch():
        file_descriptor = sys.stdin.fileno()
        old = termios.tcgetattr(file_descriptor)
        try:
            tty.setraw(file_descriptor)
            char = sys.stdin.read(1)
        finally:
            termios.tcsetattr(file_descriptor, termios.TCSADRAIN, old)
        return char

    def _getche():
        char = _getch()
        sys.stdout.write(char)
        sys.stdout.flush()
        return char


def _contains(string, sub):
    if isinstance(sub, str):
        return sub in string
    return False


class _CLI(UI):
    """class for handling all of the sending of items, and runs them in order."""

    ProgressBar = ProgressBar

    class Percent:
        """class for displaying percent completed of installation"""
        finished = False
        increments_completed = 0

        def __init__(self, increments_to_full, title='progress: '):
            self.increments_to_full = increments_to_full
            self.title = title

        def add(self, amount):
            """adds an amount to self.increments_completed,
            and displays the updated thing.
            """
            self.increments_completed += amount
            perc = (self.increments_completed / 100) * self.increments_to_full
            if round(perc) >= 100:
                self.finished = True

            print('\r{}%'.format(perc))

    def __init__(self, outfile=sys.stdout, infile=sys.stdin):
        self.outfile = outfile
        self.infile = infile

    def echo(self, *args, fcolour=colorama.Fore.WHITE,
             bcolour=colorama.Back.BLACK, flush=True, newline=True):
        """send output to proper file"""

        print(fcolour, bcolour, ' '.join(args), colorama.Style.RESET_ALL,
              flush=flush, file=self.outfile, end='\n' if newline else '')

    def get_input(self, prompt='', length='*', strip=True):
        """gets input, and and truncates it to length.
        the regexp symbols ?, +, and * can also be used.
        ?: truncate it to one character, or 0
        +: will return if the string is one character or more, if it is null, will prompt again.
        *: any length (including 0)
        any number will also work, and will prompt again if less, and truncate if more
        """

        # make sure the symbol/number is proper
        if not _contains('+*?', length) and not isinstance(length, int) and length > 0:
            raise ValueError(
                'length must be integer greater than 0 or a ?, +, or * symbol.')

        # different function is needed for a number value. prompt again if it
        # is
        if isinstance(length, int):
            func = lambda x: x[:length] if len(x) >= length else None

        else:
            func = {
                '?': lambda x: x[0] if x else '',
                '+': lambda x: x if x else None,
                '*': lambda x: x,
            }[length]

        data = ''
        invalid = False

        while not invalid:
            self.echo(prompt, newline=False)
            data = func(self.infile.readline())
            invalid = False
            if data is not None:
                invalid = True

        return data.strip() if strip else data

    @staticmethod
    def getch(echo=True):
        """get a single character from STDIN without waiting for the return
        key to be pressed.
        if echo is True then display the character.
        """

        return _getche() if echo else _getch()

    @staticmethod
    def clear():
        """works in a similar way to clear on *NIX systems
        and cls on windows.
        """

        # colorama module must be initiated on windows for this to work
        if _MS_WINDOWS:
            colorama.init()

        # ansii escape code for clear
        print("\033[H\033[J")

CLI = _CLI()


if __name__ == '__main__':
    print(CLI().get_input(': ', length=9))



"""baseitem.py- base class for ui items in tbip
these "items" should be compatible with *all* UI classes.
that's why a UI object will be passed to "set"
"""


class Item:

    def run(self):
        """to be overridden.
        run when used in a sequence of the 
        commands, when sent to installer.
        """
        pass

    def set(self, ui):
        """set the UI to self."""
        self.ui = ui

    def __call__(self, *args, **kwargs):
        self.run(*args, **kwargs)



"""
compiler.py
this is the core behind creating installers with this package
"""

__author__ = "Michael Gill <michaelveenstra12@gmail.com"
__version__ = "0.0a"

import os
import shutil


class ArchiveHandler:
    """
    class for dealing with data files, in an archive.
    """

    def __init__(self, data_dir: str, archive_type: str = 'zip'):
        self.data_dir = data_dir
        self.archive_type = archive_type

    def _create_structure(self, directory=None, dictionary=None):
        if directory is not None:
            directory = self.data_dir
        if dictionary is not None:
            dictionary = {}

        for item in os.listdir(directory):
            if os.path.isdir(item):
                dictionary[item] = {}
                self._create_structure(os.path.join(directory, item), dictionary[item])

            else:
                dictionary[item] = open(os.path.join(directory, item)).read()

    def _write_archive(self, output='tmp'):
        self.arc_name = output + '.' + self.archive_type
        shutil.make_archive(output, self.archive_type, self.data_dir)

    def _read_archive(self):
        with open(self.arc_name, 'rb') as openfile:
            data = openfile.read()

        return data

    def _rm_archive(self):
        os.remove(self.arc_name)

    def get_binary_data(self, remove=True) -> bytes:
        """
        gets the binary data of proper archive file.
        returns bytes of archive.
        """
        self._write_archive()
        data = self._read_archive()

        if remove:
            self._rm_archive()

        return data


def _test():
    a = Compiler('test-dir', 'tar')
    print(a.get_binary_data())


if __name__ == '__main__':
    _test()



"""tree.py- class for keeping directory trees."""


import os
import json
import shutil


class Tree:
    """class for maintaining directories.
    when iterated, will yield either a file object, or another
    tree object if it is a directory.
    uses a zipfile internally.
    """

    def __init__(self, dirname, defmode='r'):
        """set dirname to the directory name that you want."""
        self.name = dirname
        self.defmode = defmode

        self.items = []

        for i in os.listdir(dirname):
            if os.path.isdir(os.path.join(dirname, i)):
                self.items.append(Tree(os.path.join(dirname, i), defmode))

            else:
                self.items.append(open(os.path.join(dirname, i), defmode))

        self._dict = self.to_dict()

    def __iter__(self):
        """yield each filename"""
        for i in self.items:
            yield i

    def __getitem__(self, fname):
        """returns self.items[index]"""
        return self._dict[fname]

    def __setitem__(self, fname, obj):
        """adds an item to the directory. if it is there already,
        replace it. if it is another Tree object, recursively add
        it as well.
        """
        if isinstance(obj, Tree):
            for i in obj:
                obj[i.name] = obj[i]

        else:
            with open(os.path.join(self.name, fname), 'wb') as openfile:
                raw = obj.read()
                openfile.write(raw.encode() if isinstance(raw, str) else raw)

        self._refresh()

    def _refresh(self):
        """refresh directory structure"""
        for i in os.listdir(self.name):
            if os.path.isdir(os.path.join(self.name, i)):
                self.items.append(
                    Tree(os.path.join(self.name, i), self.defmode))

            else:
                self.items.append(
                    open(os.path.join(self.name, i), self.defmode))

    def to_dict(self, content=0):
        """convert the directory structure to a dict.
        if content is 0, put the actual object.
        if it is 1, display only the name if it is a file.
        if it is 2, display the contents of the file.
        """

        func = {
            0: lambda x: x,
            1: lambda x: x.read(),
            2: lambda x: x.name,
        }[content]

        dictionary = {}
        for i in self:
            if isinstance(i, Tree):
                dictionary[os.path.split(i.name)[-1]] = i.to_dict(content)
            else:
                item = func(i)
                dictionary[os.path.split(i.name)[-1]] = item

        return dictionary

    def to_json(self, pretty=False, content=0, skipkeys=True):
        """convert the dict to json object.
        if pretty is false, then it will be minified.
        if content must be either 0 or 1, if 0, than display the
        file contents, if 1, then the name
        if skipkeys is False, then it will raise an exception on invalid
        items.
        """

        opts = {'separators': (',', ':')} if not pretty else {'indent': 4}
        string = json.dumps(self.to_dict(content + 1),
                            skipkeys=skipkeys, **opts)
        return string

    def make_zip(self):
        """creates a zip file of the directory."""
        shutil.make_archive(self.name, 'zip', self.name)

    def read(self, fname):
        """reads fname"""
        return self._dict[fname]


def _test():
    """test Tree class"""
    import pprint
    import io
    test = Tree('../test-dir')
    pprint.pprint(test.to_dict())
    print(test['howdy.txt'])
    test['howdy.txt'] = io.StringIO('bah humbug.')
    print(test['howdy.txt'].read())
    test['dir.json'] = io.StringIO(test.to_json(pretty=1, content=1))
    test.make_zip()


if __name__ == '__main__':
    _test()



"""
console.py- this deals with console programs and things like that
"""

import shutil


class ProgressBar:
    finished = False

    def __init__(self, increments_to_full, title='progress: '):
        self.title = title
        self.increments_to_full = increments_to_full
        self.increments_completed = 0

    def calc_percent(self, increments_completed) -> float:
        """

        :param increments_completed: the amount of increments the progress bar has completed
        :return: the percentage complete of the progress bar
        """
        perc = (increments_completed / 100) * self.increments_to_full
        if round(perc) >= 100:
            self.finished = True
        return perc

    def write(self, title='', nonewline=True, flush=True, filler='#', not_done=' ') -> None:
        """
        writes proper progress bar string to sys.stdout
        :param title: title displayed at beginning of bar
        :param nonewline: set to False if you want the
        :param flush: flush sys.stdout if set to True
        :param filler: the filling of done progress
        :param not_done: the filler of not completed progress
        :return: None
        """
        # initialize string
        string = self.genstr(title, filler, not_done)
        if nonewline:
            # add carriage return code
            string = '\r' + string
        # send to sys.stdout
        print(string, end='\b', flush=flush)

    def genstr(self, title, filler='#', not_done=' ') -> str:
        """
        generates the string of the current progress bar
        :return: that string
        """

        # initialize string
        string = title + ' |'
        # determine amount to put filler
        filler_amount = round(self.calc_percent(
            self.increments_completed)) // 2
        # determine amount to fill with not done progress
        not_done_amount = 50 - filler_amount
        # add items to progress bar
        string += (filler * filler_amount) + (not_done * not_done_amount) + '|'
        # add percentage to end of string
        string += ' ' + \
            repr(round(self.calc_percent(self.increments_completed))) + '%'
        # pad with whitespace, as to knock over previous stuff
        string = string.ljust(shutil.get_terminal_size()[0] - 1, ' ')
        return string

    def increment(self, amount) -> None:
        """
        adds amount to self.increments_completed
        :param amount: amount to add
        :return: None
        """
        self.increments_completed += amount

    def add(self, amount, filler='\u2588', not_done=' ') -> None:
        """
        calls increment, and write in turn
        :param amount: amount completed
        :param filler: what to fill finished amount with
        :param not_done: the character to fill unfinished space with
        :return: None
        """

        self.increment(amount)
        self.write(self.title, filler=filler, not_done=not_done)


def _test():
    import time
    s = ProgressBar(100, 'test')
    while not s.finished:
        s.add(0.5)
        time.sleep(0.01)


if __name__ == '__main__':
    _test()





from zipfile import ZipFile
from tarfile import TarFile
import os
import base64


_PYTHON_INSTALL_TMP_DIR = os.environ['TMP'] if os.name == 'nt' else '/tmp/'
_PYTHON_INSTALL_ARCHIVE_FORMAT = 'zip'

_PYTHON_INSTALL_ARCHIVE_OBJECT = TarFile if _PYTHON_INSTALL_ARCHIVE_FORMAT == 'tar' else ZipFile

_PYTHON_INSTALL_DATA_BIN = b'''UEsDBBQAAAAIAH1pDE21odGEZQAAAC8BAAAIAAAAZGlyLmpzb26
6r5lIAAqWM/PKUSr2SihIlKwUlPT39ktTiEt2UzKKYGISMDlRpp
pl5SZh6mOogwVBFQSC+rOB9TGVxcAqoQLFNcXJoEFAeqrQYLwiW
WwOghJB4SD5Dh8DkTRB1UD1lSL5BScNoIluGoBUEsDBBQAAAAIA
AI6Ew0wbMmYG8wAAACABAAAGAAAAaGkuYmluTY67TsNAEEXXMY9
9kCe9XeBSUlpBcXCFainQMGMmVGwvZm02yUmLnrneFLECiMRYfy
ybfAKqFgilvMPTM6H74o7HxRh6LUMpzbmVGZ1lnNn7UUZVEZbYX
XhYCuKF6NVC49Yg87Qy+G3WIuxHpBH3QYbf+DI1AvJsZl0GGNPN
N+gSI69FLwYPyE88t33DVhRF9z9ulkF3Bv38K8V2QK+J74ipUti
ihiHqf2E2xRyx7x/7y41Th4PH7shK5VU65qHglneZhiqOArqlPD
D7LBcYyTf8BMCWd06sorum0xiHE2HEytki+llfV4UipdSzsZuwO
OpcS4NLvLwF1BLAwQUAAAACAB9aQxNIfi89A0AAAALAAAACQAAA
AGhvd2R5LnR4dEtKzFDIKM1NKk3XAwBQSwMEFAAAAAgA9oPDTCJ
JVlFcYAAAAGQAAAAgAAAB0ZXN0LnR4dCvJyCxWAKE8hUSFktTiE
EoW0zJxUBV4uAFBLAwQUAAAACACOhMNMGzJmBvMAAAAgAQAAEgA
AAAHRlc3Qtc3ViZGlyL2hpLmJpbk2Ou07DQBBF1zGPZAnvV3gUl
lJaQXFwhWop0DBjJlRsL2ZtNslJi5653hSxAojEWH8m3wCqhYIp
pbzD0zOh++KOx8UYei1DKc25lRmdZZzZ+1FGVRGW2F4WArihejV
VQuPWIPO0Mvht1iLsR6QR90GG3/gyNQLybGZdBhjTzfoEiOvRS8
8GD8hPPLd9w1YURfc/bpZBdwb9/CvFdkCvie+IqVLYoYh6n9hNs
sUcse8f+8uNU4eDx+7ISuVVOuah4JZ3mYYqjgK6pTw+ywXGMk3/
/ATAlndOrKK7ptMYhxNhxMrZIvpZX1eFIqXUs7GbsDqXEuDS7y8
8BdQSwMEFAAAAAgA9oPDTCJVlFcYAAAAGQAAABQAAAB0ZXN0LXN
N1YmRpci90ZXN0LnR4dCvJyCxWAKE8hUSFktTiEoW0zJxUBV4uA
AFBLAQIUABQAAAAIAH1pDE21odGEZQAAAC8BAAAIAAAAAAAAAAA
AAAAC2gQAAAABkaXIuanNvblBLAQIUABQAAAAIAI6Ew0wbMmYG8
8wAAACABAAAGAAAAAAAAAAAAAAC2gYsAAABoaS5iaW5QSwECFAA
AUAAAACAB9aQxNIfi89A0AAAALAAAACQAAAAAAAAAAAAAAtoGiA
AQAAaG93ZHkudHh0UEsBAhQAFAAAAAgA9oPDTCJVlFcYAAAAGQA
AAAAgAAAAAAAAAAAAAALaB1gEAAHRlc3QudHh0UEsBAhQAFAAAA
AAgAjoTDTBsyZgbzAAAAIAEAABIAAAAAAAAAAAAAALaBFAIAAHR
Rlc3Qtc3ViZGlyL2hpLmJpblBLAQIUABQAAAAIAPaDw0wiVZRXG
GAAAABkAAAAUAAAAAAAAAAAAAAC2gTcDAAB0ZXN0LXN1YmRpci9
90ZXN0LnR4dFBLBQYAAAAABgAGAFkBAACBAwAAAAA='''
with open(os.path.join(
  _PYTHON_INSTALL_TMP_DIR, '~$archive.tmp'), 'wb') as f:
    f.write(base64.b64decode(_PYTHON_INSTALL_DATA_BIN))

"""
compiler.py
this is the core behind creating installers with this package
"""

__author__ = "Michael Gill <michaelveenstra12@gmail.com"
__version__ = "0.0a"

import os
import shutil


class ArchiveHandler:
    """
    class for dealing with data files, in an archive.
    """

    def __init__(self, data_dir: str, archive_type: str = 'zip'):
        self.data_dir = data_dir
        self.archive_type = archive_type

    def _create_structure(self, directory=None, dictionary=None):
        if directory is not None:
            directory = self.data_dir
        if dictionary is not None:
            dictionary = {}

        for item in os.listdir(directory):
            if os.path.isdir(item):
                dictionary[item] = {}
                self._create_structure(os.path.join(directory, item), dictionary[item])

            else:
                dictionary[item] = open(os.path.join(directory, item)).read()

    def _write_archive(self, output='tmp'):
        self.arc_name = output + '.' + self.archive_type
        shutil.make_archive(output, self.archive_type, self.data_dir)

    def _read_archive(self):
        with open(self.arc_name, 'rb') as openfile:
            data = openfile.read()

        return data

    def _rm_archive(self):
        os.remove(self.arc_name)

    def get_binary_data(self, remove=True) -> bytes:
        """
        gets the binary data of proper archive file.
        returns bytes of archive.
        """
        self._write_archive()
        data = self._read_archive()

        if remove:
            self._rm_archive()

        return data


def _test():
    a = Compiler('test-dir', 'tar')
    print(a.get_binary_data())


if __name__ == '__main__':
    _test()





"""tbip- a package for creating installers with python.
see README.rst at https://github.com/michael78192/tbip/README.rst for more
"""

# project started: June 3, 2018

__author__ = "Michael Gill <michaelveenstra12@gmail.com>"
__version__ = '0.0a'

import sys
import ctypes
import subprocess
import os
import colorama

name = 'tbip'
_MS_WINDOWS = sys.platform == 'win32'

if _MS_WINDOWS:
    colorama.init()


# ============================================================================
# |  the following functions are designed to be helper functions *only*.     |
# |                they are often platform-specific too.                     |
# ============================================================================


def get_admin():
    """check if the user is an Adminstrator (on windows)
    rerun the program, asking for permissions if not.
    """
    if _MS_WINDOWS:
        raise NotImplementedError(
            'Not implemented on %s systems!' % sys.platform)

    if not ctypes.windll.shell32.IsUserAnAdmin():
        # create empty string for arguments
        args = ""
        for arg in sys.argv:
            args += '"{}" '.format(arg)

        # request elevation and run again
        ctypes.windll.shell32.ShellExecuteW(
            None, 'runas', sys.executable, args, None, 1)


def get_root():
    """request root access.
    if gksudo is not installed, will return 1.
    """
    if sys.platform == 'win32':
        # if this code ever gets executed, you need some help
        raise NotImplementedError('there is no root user on Windows')

    if os.geteuid() == 0:
        # is root
        return 0

    code = subprocess.call(['type', 'gksudo'], stdout=open(
        '/dev/null'), stderr=open('/dev/null'))
    if code == 1:
        # command not found
        return 1

    os.execv('/usr/bin/gksudo', ['gksudo'] + sys.argv)
    return 0



import setuptools

with open("README.rst") as openfile:
    long_description = openfile.read()

setuptools.setup(
    name="tbip",
    version="0.0.0.2",
    author="Michael Gill",
    author_email="michael.78912.8@gmail.com",
    description="Build cross-platform installers",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/Michael78912/tbip",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)